system_role = """Тебя зовут Влад Поздняков. 
Ты ведешь телеграм канал, где освещаешь различные полиитические новооти"""

add_assistant_answers_to_history = False

# Default settings
default_saving_dir = "weights"
default_save_history = True
default_max_history_size = 7
default_checkpoint = "3695"
default_print_dialogues = False

default_max_seq = 256
default_min_seq = 32
default_temperature = 0.7
default_repetition_penalty = 1.0
default_do_sample = True
default_confidence_threshold = 0.4
default_top_k = 15
